﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

public class GameManager : MonoBehaviour
{
    public enum DeathType
    {
        Lava,
        Floor
    }

    public enum GameState { READY, GAME, PAUSED, RESULTS }
    public static GameState currentState;

    public static GameManager instance;

    public static string axisHorizontal = "HorizontalPlayer";
    public static string axisVertical = "VerticalPlayer";
    public static string keyJump = "JumpPlayer";
    public static string keyDive = "DivePlayer";
    public static string start = "StartPlayer";

    public static string tagSpawnPoints = "Respawn";

    private GameObject[] spawnPoints;

    public static List<Player> players = new List<Player>();
    public List<PlayerController> playerControllers = new List<PlayerController>();

    private int spawnIndex = 0;

    private Dictionary<int, int> mappedSkins;

    public static bool allReady
    {
        get
        {
            for (int i = 0; i < players.Count; i++)
            {
                if (!players[i].isReady) { return false; }
            }
            return true;
        }
    }

    [SerializeField]
    private Material deadDuckMaterial;

    [SerializeField]
    private GameObject playerPrefab;

    [SerializeField]
    private float respawnTime;

    [SerializeField]
    private PLayerMaterials[] playerMaterials;

    public int numberOfLifes = 1;

    private void Awake()
    {
        if (instance == null) { instance = this; }
        else if (instance != this) { Destroy(gameObject); }
        DontDestroyOnLoad(gameObject);
    }

    void Start()
    {
        mappedSkins = new Dictionary<int, int>();

        spawnPoints = GameObject.FindGameObjectsWithTag(tagSpawnPoints);
        if (spawnPoints.Length == 0) { Debug.LogError("No spawnpoints found in this scene!"); }

        SwitchState(GameState.READY);
    }


    void Update()
    {
        if (currentState == GameState.READY)
        {
            foreach (KeyCode kcode in Enum.GetValues(typeof(KeyCode)))
            {
                if (Input.GetKeyDown(kcode))
                {
                    string key = kcode.ToString();
                    Debug.Log("key pressed " + key);
                    if (key.Contains("Joystick") && key.Contains("Button0"))
                    {
                        key = key.Replace("Button0", string.Empty);
                        key = key.Replace("Joystick", string.Empty);
                        if (key != string.Empty)
                        {
                            int index = int.Parse(key);

                            bool shouldSpawn = false;
                            if (players.Count == 0) { shouldSpawn = true; }
                            else
                            {
                                bool alreadyTaken = false;
                                foreach (Player p in players)
                                {
                                    if (p.joyNumber == index)
                                    {
                                        alreadyTaken = true;
                                        break;
                                    }
                                }
                                if (!alreadyTaken) { shouldSpawn = true; }
                            }
                            if (shouldSpawn) { AddPlayer(int.Parse(key)); }
                        }
                    }
                }
            }

            for (int i = 0; i < players.Count; i++)
            {
                if (Input.GetButtonDown(start + players[i].joyNumber))
                {
                    // if all are ready, then we can go ahead and start the game
                    if (allReady)
                    {
                        AudioManager.StopMusic();
                        AudioManager.PlayMusic(AudioManager.instance.gameMusicClip, true);
                        SwitchState(GameState.GAME);
                        WaveSpawner.instance.StartSpawningWaves();
                        return;
                    }

                    players[i].isReady = true;
                    UIManager.TogglePlayerReady(i, true);
                }

                if (Input.GetButtonDown(keyJump + players[i].joyNumber))
                {
                    players[i].isReady = true;
                    UIManager.TogglePlayerReady(i, true);
                }

                // players can cancel their ready state
                if (Input.GetButtonDown(keyDive + players[i].joyNumber))
                {
                    players[i].isReady = false;
                    UIManager.TogglePlayerReady(i, false);
                }
            }
        }
        else if (currentState == GameState.RESULTS)
        {
            for (int i = 0; i < players.Count; i++)
            {
                if (Input.GetButtonDown(start + players[i].joyNumber)) { SwitchState(GameState.READY); }
            }
        }
    }

    public void SwitchState(GameState state)
    {
        currentState = state;

        switch (currentState)
        {
            case GameState.READY:
                players = new List<Player>();

                AudioManager.StopMusic();
                AudioManager.PlayMusic(AudioManager.instance.menuClip, true);

                UIManager.SwitchState(UIManager.UIState.READY);
                break;
            case GameState.GAME:
                for (int i = 0; i < instance.playerControllers.Count; i++) { Destroy(instance.playerControllers[i].gameObject); }
                instance.playerControllers = new List<PlayerController>();
                for (int i = 0; i < players.Count; ++i)
                {
                    instance.SpawnPlayers(players[i].joyNumber, i);
                }
                LifesManager.instance.Init(players.Count, instance.numberOfLifes);
                UIManager.SwitchState(UIManager.UIState.MAIN);
                break;
            case GameState.PAUSED:
                break;
            case GameState.RESULTS:
                WaveSpawner.instance.StopSpawningWaves();
                //StartCoroutine(PlayEndOfGameSounds();
                foreach (PlayerController pc in instance.playerControllers) { if (pc.state == PlayerController.State.Alive) { UIManager.instance.AdjustResultsText(pc); } }
                UIManager.SwitchState(UIManager.UIState.RESULTS);

                AudioManager.StopMusic();
                AudioManager.PlayMusic(AudioManager.instance.gameOverClip);

                break;
            default:
                break;
        }
    }

    /*public IEnumerator PlayEndOfGameSounds()
    {
        AudioManager.
    }*/

    public Material GetDeadDuckMaterial()
    {
        return deadDuckMaterial;
    }

    public Vector3 GetRandomSpawnPosition()
    {
        spawnIndex = (spawnIndex + 1) % spawnPoints.Length;
        return spawnPoints[spawnIndex].transform.position;
    }

    private void AddPlayer(int joyNum)
    {
        players.Add(new Player(joyNum));
        UIManager.instance.AddPlayer();

        if (!mappedSkins.ContainsKey(joyNum))
        {
            int index = UnityEngine.Random.Range(0, 8);
            while (mappedSkins.ContainsValue(index))
            {
                index++;
                index = index % 8;
            }
            mappedSkins.Add(joyNum, index);
        }
        AudioManager.PlayClip(AudioManager.instance.quaks[mappedSkins[joyNum]]);
    }

    public int GetSkinIndex(int index)
    {
        return mappedSkins[players[index].joyNumber];
    }

    public int GetSkinIndex(PlayerController controller)
    {
        return mappedSkins[controller.playerJoypadNumber];
    }

    private void ReadyPlayer()
    {

    }

    private void SpawnPlayers(int joyNum, int id)
    {
        PlayerController controller = Instantiate(playerPrefab).GetComponent<PlayerController>();
        controller.transform.position = GetRandomSpawnPosition();
        controller.numberOfLifesLeft = numberOfLifes;
        controller.playerJoypadNumber = joyNum;
        controller.playerMaterials = playerMaterials[GetSkinIndex(controller)];
        playerControllers.Add(controller);
    }

    private void PlayDeathSequence(PlayerController player, DeathType deathType)
    {
        AudioManager.PlayClip(AudioManager.instance.floorDeath);
        AudioManager.PlayClip(AudioManager.instance.lavaDeath);
        switch (deathType)
        {
            case DeathType.Lava:
                player.BlastAwayAndKill();
                break;
            case DeathType.Floor:
                player.SubmergeAndKill();
                break;
        }
    }

    public void KillPlayer(PlayerController player, DeathType deathType)
    {
        if (player.state == PlayerController.State.Alive)
        {
            PlayDeathSequence(player, deathType);

            // we will not respawn the player, but mark him as dead
            StartCoroutine(WaitAndRespawn(player));
        }
    }

    public void CheckVictoryConditions(PlayerController player)
    {
        player.numberOfLifesLeft--;
        LifesManager.instance.UpateUI(playerControllers.IndexOf(player), player.numberOfLifesLeft);
        if (player.numberOfLifesLeft == 0)
        {
            CheckVictoryConditions();
        }
    }

    private IEnumerator WaitAndRespawn(PlayerController player)
    {
        yield return new WaitForSeconds(respawnTime);

        if (player.numberOfLifesLeft > 0)
        {
            AudioManager.PlayClip(AudioManager.instance.quaks[GetSkinIndex(player)]);
            Vector3 position = GetRandomSpawnPosition();
            player.Respawn(position);
        }
    }

    public static void CheckVictoryConditions()
    {
        int totalAlive = 0;
        foreach (var item in instance.playerControllers)
        {
            if (item.numberOfLifesLeft > 0) { totalAlive++; }
        }
        if (totalAlive == 1)
        {
            Debug.Log("The last man was standing!");
            instance.SwitchState(GameState.RESULTS);
        }
        else if (instance.playerControllers.Count == 1)
        {
            Debug.Log("This player played solo... hardcore! Either way, he is dead.");
            instance.SwitchState(GameState.RESULTS);
        }
        else
        {
            Debug.LogFormat("There are still {0} players alive, fighting!", totalAlive);
        }
    }
}

public class Player
{
    public int joyNumber { get; private set; }
    public bool isReady { get; set; }

    public Player(int joyNum)
    {
        joyNumber = joyNum;
    }
}