﻿using System.Collections;
using System.Collections.Generic;
using System;
using UnityEngine;

[Serializable]
public struct PLayerMaterials
{
	public Material idleMaterial;
	public Material inBetweenMaterial;
	public Material speedMaterial;
};

public class PlayerController : MonoBehaviour
{
	public enum State
	{
		Alive,
		DyingFloor,
		DyingWave,
		Dead}

	;

	public int playerJoypadNumber = 1;

	public State state = State.Alive;
	public float moveAmount;
	public float freeMoveAmount = 1;
	public float gravityAmount;
	public float jumpAmount;
	public int numberOfLifesLeft;

	public bool isInWater = false;
	private float jumpSlack = 2.0f;

	public float maxJumpTime = 1.0f;
	private float currentJumpTime = 0.0f;

	private bool isMovingLeft = false;
    
	private Quaternion originalRot;
	private Renderer duckyRenderer;

	public PLayerMaterials playerMaterials;

	public ParticleSystem splashParticles;
	public ParticleSystem bubbleBoostParticles;
	public ParticleSystem bubbleBoostBurstParticles;

	Rigidbody rb;

	void Start ()
	{
		rb = GetComponent<Rigidbody> ();
		duckyRenderer = GetComponentInChildren<Renderer> ();

		duckyRenderer.material = playerMaterials.idleMaterial;
        
		originalRot = transform.rotation;
	}

	void Update ()
	{
		if (state == State.Alive) {
			HandleInput ();
		}        
	}

	private void HandleInput ()
	{
		transform.rotation = originalRot;

		// handle gravity
		if (!isInWater) {
			rb.AddForce (Vector3.down * gravityAmount);
		} else {
			rb.AddForce (Vector3.up * gravityAmount);
		}

		// handle left/right
		// Slow down movement when duck is in the air
		if (Input.GetAxis (GameManager.axisHorizontal + playerJoypadNumber) > 0) {
			isMovingLeft = false;
			rb.AddForce (Vector3.right * (isInWater ? moveAmount : freeMoveAmount));
			HandleRotation ();
		} else if (Input.GetAxis (GameManager.axisHorizontal + playerJoypadNumber) < 0) {
			isMovingLeft = true;
			rb.AddForce (Vector3.left * (isInWater ? moveAmount : freeMoveAmount));
			HandleRotation ();
		}

		// handle jump/dive
		if (Input.GetButtonDown (GameManager.keyJump + playerJoypadNumber) && currentJumpTime <= 0.0f) {
			bool allowedToJump = false;

			RaycastHit hit;
			if (Physics.Raycast (new Ray (transform.position, Vector3.down), out hit)) {
				if (Vector3.Distance (transform.position, hit.point) <= jumpSlack) {
 					Debug.DrawLine (transform.position, hit.point, Color.green, 5.0f);
					bubbleBoostBurstParticle (transform.position);
					bubbleBoostParticle (transform.position);
					allowedToJump = true;
				}
			}

			// we can jump if we are in water, or just above it, as calculated by the raycast
			if (isInWater || allowedToJump) {
				var horiz = Input.GetAxis (GameManager.axisHorizontal + playerJoypadNumber);
				var vert = Input.GetAxis (GameManager.axisVertical + playerJoypadNumber);

				rb.AddForce (horiz * jumpAmount, vert * jumpAmount, 0);
				currentJumpTime = maxJumpTime;

                AudioManager.PlayClip(AudioManager.instance.duckRocketJump);
				StartCoroutine (BoostSequence ());
			}
			HandleRotation ();
		}

		// we may dive if we are under water
		if (Input.GetButtonDown (GameManager.keyDive + playerJoypadNumber) && isInWater) {
			rb.AddForce (Vector3.down * jumpAmount);
			HandleRotation ();
		}

		// handle jump time
		currentJumpTime -= Time.deltaTime;

		if (isMovingLeft)
			transform.Rotate (Vector3.forward, 180);
	}

	private IEnumerator BoostSequence ()
	{
		duckyRenderer.material = playerMaterials.inBetweenMaterial;
        
		yield return new WaitForSeconds (.2f);

		if (state == State.Alive)
			duckyRenderer.material = playerMaterials.speedMaterial;
		yield return new WaitForSeconds (.3f);

		if (state == State.Alive)
			duckyRenderer.material = playerMaterials.idleMaterial;     
	}

	private void HandleRotation ()
	{

		Vector2 direction = rb.velocity.normalized;
		float angle = Mathf.Lerp (0, Mathf.Atan2 (direction.y, direction.x) * Mathf.Rad2Deg, Time.deltaTime * 3f);
        
		transform.Rotate (Vector3.up, angle);
        
	}

	public void Kill (GameManager.DeathType deathType)
	{
		GameManager.instance.KillPlayer (this, deathType);
	}

	public void SubmergeAndKill ()
	{
		this.state = State.DyingFloor;
		if (gameObject.GetComponent<SubmergeAndKill> () == null)
			gameObject.AddComponent<SubmergeAndKill> ();        
	}

	public void BlastAwayAndKill ()
	{
		this.state = State.DyingWave;
		if (gameObject.GetComponent<BlastAwayAndKill> () == null)
			gameObject.AddComponent<BlastAwayAndKill> ();
	}

	public void Respawn (Vector3 position)
	{
		transform.rotation = originalRot;
		duckyRenderer.material = playerMaterials.idleMaterial;
		state = State.Alive;
		transform.position = position;
		rb.velocity = Vector3.zero;
		rb.angularVelocity = Vector3.zero;
		currentJumpTime = maxJumpTime;
	}

	private void startParticleSystem (ParticleSystem particleSystem, Vector3 position)
	{
		ParticleSystem newParticleSystem = Instantiate (
			                                   particleSystem,
			                                   position,
			                                   Quaternion.identity
		                                   ) as ParticleSystem;

		// Make sure it will be destroyed
		Destroy (
			newParticleSystem.gameObject,
			newParticleSystem.startLifetime
		);
	}

	public void splashParticle (Vector3 position)
	{
		startParticleSystem (splashParticles, position);
	}

	public void bubbleBoostParticle (Vector3 position)
	{
		startParticleSystem (bubbleBoostParticles, position);
	}

	public void bubbleBoostBurstParticle (Vector3 position)
	{
		startParticleSystem (bubbleBoostBurstParticles, position);
	}
}
